@@ -28,6 +28,9 @@
 #include "cif++/compound.hpp"
 #include "cif++/row.hpp"
 
+#include <stdexcept>
+#include <string>
+
 // --------------------------------------------------------------------
 
 namespace cif::pdb
@@ -187,6 +190,8 @@ void checkEntities(datablock &db)
 
 void createEntityIDs(datablock &db)
 {
+	using namespace literals;
+
 	// Suppose the file does not have entity ID's. We have to make up some
 
 	// walk the atoms. For each auth_asym_id we have a new struct_asym.
@@ -198,6 +203,7 @@ void createEntityIDs(datablock &db)
 	// that should cover it
 
 	auto &atom_site = db["atom_site"];
+	auto &entity = db["entity"];
 	auto &cf = compound_factory::instance();
 
 	std::vector<std::vector<row_handle>> entities;
@@ -206,16 +212,29 @@ void createEntityIDs(datablock &db)
 	int lastSeqID = -1;
 	std::vector<row_handle> waters;
 
-	for (auto rh : atom_site)
+	int nextEntityID;
+	try
+	{
+		if (entity.empty())
+			nextEntityID = 1;
+		else
+			nextEntityID = entity.find_max<int>("id") + 1;
+	}
+	catch (...)
+	{
+		nextEntityID = 1;
+	}
+
+	for (auto rh : atom_site.find("label_entity_id"_key == cif::null))
 	{
 		residue_key_type k = rh.get<std::optional<std::string>,
-			 std::optional<int>,
-			 std::optional<std::string>,
-			 std::optional<std::string>,
-			 std::optional<int>,
-			 std::optional<std::string>>(
-			 "auth_asym_id", "auth_seq_id", "auth_comp_id",
-			 "label_asym_id", "label_seq_id", "label_comp_id");
+			std::optional<int>,
+			std::optional<std::string>,
+			std::optional<std::string>,
+			std::optional<int>,
+			std::optional<std::string>>(
+			"auth_asym_id", "auth_seq_id", "auth_comp_id",
+			"label_asym_id", "label_seq_id", "label_comp_id");
 
 		std::string comp_id = get_comp_id(k);
 
@@ -230,8 +249,8 @@ void createEntityIDs(datablock &db)
 
 		bool is_monomer = cf.is_monomer(comp_id);
 
-		if (lastAsymID == asym_id and lastSeqID == seq_id and not is_monomer)
-			continue;
+		// if (lastAsymID == asym_id and lastSeqID == seq_id and not is_monomer)
+		// 	continue;
 
 		if (asym_id != lastAsymID or (not is_monomer and lastSeqID != seq_id))
 			entities.push_back({});
@@ -243,6 +262,7 @@ void createEntityIDs(datablock &db)
 	}
 
 	std::map<std::size_t, std::string> entity_ids;
+	std::map<std::string, std::string> newEntitiesForCompound;
 
 	atom_site.add_item("label_entity_id");
 
@@ -251,7 +271,37 @@ void createEntityIDs(datablock &db)
 		if (entity_ids.contains(i))
 			continue;
 
-		auto entity_id = std::to_string(i + 1);
+		residue_key_type k = entities[i].front().get<std::optional<std::string>, std::optional<int>, std::optional<std::string>, std::optional<std::string>, std::optional<int>, std::optional<std::string>>(
+			"auth_asym_id", "auth_seq_id", "auth_comp_id",
+			"label_asym_id", "label_seq_id", "label_comp_id");
+
+		std::string comp_id = get_comp_id(k);
+
+		std::string entity_id;
+		if (auto v = db["pdbx_entity_nonpoly"].find_first("comp_id"_key == comp_id); v)
+			entity_id = v.get<std::string>("entity_id");
+		else if (auto i2 = newEntitiesForCompound.find(comp_id); i2 != newEntitiesForCompound.end())
+			entity_id = i2->second;
+		else
+		{
+			entity_id = std::to_string(nextEntityID++);
+
+			if (cf.is_monomer(comp_id))
+				entity.emplace({ //
+					{ "id", entity_id },
+					{ "type", "polymer" } });
+			else if (cf.is_water(comp_id))
+				entity.emplace({ //
+					{ "id", entity_id },
+					{ "type", "water" } });
+			else
+				entity.emplace({ //
+					{ "id", entity_id },
+					{ "type", "non-polymer" } });
+
+			newEntitiesForCompound[comp_id] = entity_id;
+		}
+
 		entity_ids[i] = entity_id;
 
 		for (std::size_t j = i + 1; j < entities.size(); ++j)
@@ -439,7 +489,7 @@ void checkAtomRecords(datablock &db)
 	// And negative seq_id values
 	if (atom_site.contains(key("label_seq_id") < 0))
 		fixNegativeSeqID(atom_site);
-	
+
 	std::set<std::string> polymer_entities;
 	if (db["entity"].empty())
 	{
@@ -576,7 +626,7 @@ void checkAtomRecords(datablock &db)
 			row["label_atom_id"] = row["auth_atom_id"].text();
 		else if (row["auth_atom_id"].empty())
 			row["auth_atom_id"] = row["label_atom_id"].text();
-		
+
 		// Rewrite the coordinates and other items that look better in a fixed format
 		// Be careful not to nuke invalidly formatted data here
 		for (auto [item_name, prec] : std::vector<std::tuple<std::string_view, int>>{
@@ -690,23 +740,53 @@ void checkAtomAnisotropRecords(datablock &db)
 	}
 }
 
-void createStructAsym(datablock &db)
+void checkStructAsym(datablock &db)
 {
 	auto &atom_site = db["atom_site"];
 	auto &struct_asym = db["struct_asym"];
 
-	for (const auto &[label_asym_id, entity_id] : atom_site.rows<std::string, std::string>("label_asym_id", "label_entity_id"))
+	if (struct_asym.empty())
 	{
-		if (label_asym_id.empty())
-			throw std::runtime_error("File contains atom_site records without a label_asym_id");
-		if (struct_asym.count(key("id") == label_asym_id) == 0)
+		for (const auto &[label_asym_id, entity_id] : atom_site.rows<std::string, std::string>("label_asym_id", "label_entity_id"))
 		{
-			struct_asym.emplace({
-				// clang-format off
-				{ "id", label_asym_id },
-				{ "entity_id", entity_id }
-				//clang-format on
-			});
+			if (label_asym_id.empty())
+				throw std::runtime_error("File contains atom_site records without a label_asym_id");
+			if (struct_asym.count(key("id") == label_asym_id) == 0)
+			{
+				struct_asym.emplace({
+					// clang-format off
+					{ "id", label_asym_id },
+					{ "entity_id", entity_id }
+					//clang-format on
+				});
+			}
+		}
+	}
+	else
+	{
+		for (const auto &[label_asym_id, entity_id] :
+			atom_site.rows<std::string, std::string>("label_asym_id", "label_entity_id"))
+		{
+			if (label_asym_id.empty())
+				throw std::runtime_error("File contains atom_site records without a label_asym_id");
+			
+			auto sa = struct_asym.find_first(key("id") == label_asym_id);
+			if (sa)
+			{
+				if (sa["entity_id"].empty())
+					sa.assign("entity_id", entity_id, false, true);
+				else if (sa.get<std::string>("entity_id") != entity_id)
+					throw std::runtime_error("Inconsistent entity ID's in struct_asym");
+			}
+			else
+			{
+				struct_asym.emplace({
+					// clang-format off
+					{ "id", label_asym_id },
+					{ "entity_id", entity_id }
+					//clang-format on
+				});
+			}
 		}
 	}
 }
@@ -863,11 +943,11 @@ void createEntityPoly(datablock &db)
 					else
 						letter = '(' + comp_id + ')';
 				}
-				else if (auto i = compound_factory::kAAMap.find(comp_id); i != compound_factory::kAAMap.end())
+				else if (auto i2 = compound_factory::kAAMap.find(comp_id); i2 != compound_factory::kAAMap.end())
 				{
 					c_type = "polypeptide(L)";
 
-					letter = letter_can = i->second;
+					letter = letter_can = i2->second;
 				}
 				else if (iequals(c->type(), "D-PEPTIDE LINKING"))
 				{
@@ -1425,7 +1505,7 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 	checkChemCompRecords(db);
 
 	// If the data is really horrible, it might not contain entities
-	if (not db["atom_site"].find_first(key("label_entity_id") != null))
+	if (db["atom_site"].find_first(key("label_entity_id") == null))
 		createEntityIDs(db);
 
 	// Now see if atom records make sense at all
@@ -1569,9 +1649,8 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 		checkAtomAnisotropRecords(db);
 
 	// Now create any missing categories
-	// Next make sure we have struct_asym records
-	if (auto cat = db.get("struct_asym"); cat == nullptr or cat->empty())
-		createStructAsym(db);
+	// Next make sure we have good struct_asym records
+	checkStructAsym(db);
 
 	if (auto cat = db.get("entity"); cat == nullptr or cat->empty())
 		createEntity(db);
@@ -1680,9 +1759,8 @@ void fixup_pdbx(file &file, const validator &validator)
 	db.set_validator(&validator);
 
 	// Now create any missing categories
-	// Next make sure we have struct_asym records
-	if (auto cat = db.get("struct_asym"); cat == nullptr or cat->empty())
-		createStructAsym(db);
+	// Next make sure we have good struct_asym records
+	checkStructAsym(db);
 
 	if (auto cat = db.get("entity"); cat == nullptr or cat->empty())
 		createEntity(db);
